@@ -1,13 +1,10 @@
 """
 A module implementing EOPatch merging utility
 
-Credits:
-Copyright (c) 2018-2020 William Ouellette
-Copyright (c) 2017-2022 Matej Aleksandrov, Matej Batič, Grega Milčinski, Domagoj Korais, Matic Lubej (Sinergise)
-Copyright (c) 2017-2022 Žiga Lukšič, Devis Peressutti, Nejc Vesel, Jovan Višnjić, Anže Zupanc (Sinergise)
+Copyright (c) 2017- Sinergise and contributors
+For the full list of contributors, see the CREDITS file in the root directory of this source tree.
 
-This source code is licensed under the MIT license found in the LICENSE
-file in the root directory of this source tree.
+This source code is licensed under the MIT license, see the LICENSE file in the root directory of this source tree.
 """
 from __future__ import annotations
 
@@ -20,13 +17,14 @@ from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Sequence,
 import numpy as np
 import pandas as pd
 from geopandas import GeoDataFrame
+from typing_extensions import Literal
 
 from sentinelhub import BBox
 
 from .constants import FeatureType
 from .exceptions import EORuntimeWarning
-from .utils.parsing import FeatureParser, FeatureSpec, FeaturesSpecification
-from .utils.types import Literal
+from .types import FeatureSpec, FeaturesSpecification
+from .utils.parsing import FeatureParser
 
 if TYPE_CHECKING:
     from .eodata import EOPatch
@@ -65,7 +63,7 @@ def merge_eopatches(
         - 'mean': Join arrays by taking mean values. Ignore NaN values.
         - 'median': Join arrays by taking median values. Ignore NaN values.
 
-    :return: Merged EOPatch
+    :return: Contents of a merged EOPatch
     """
     reduce_timestamps = time_dependent_op != "concatenate"
     time_dependent_operation = _parse_operation(time_dependent_op, is_timeless=False)
@@ -81,7 +79,7 @@ def merge_eopatches(
     for feature in all_features:
         feature_type, feature_name = feature
 
-        if feature_type.is_raster():
+        if feature_type.is_array():
             if feature_type.is_temporal():
                 eopatch_content[feature] = _merge_time_dependent_raster_feature(
                     eopatches, feature, time_dependent_operation, order_mask_per_eopatch, optimize_raster_temporal
@@ -92,15 +90,14 @@ def merge_eopatches(
         if feature_type.is_vector():
             eopatch_content[feature] = _merge_vector_feature(eopatches, feature)
 
-        if feature_type is FeatureType.TIMESTAMP:
+        if feature_type is FeatureType.TIMESTAMPS:
             eopatch_content[feature] = timestamps
 
         if feature_type is FeatureType.META_INFO:
             feature_name = cast(str, feature_name)  # parser makes sure of it
             eopatch_content[feature] = _select_meta_info_feature(eopatches, feature_name)
 
-        if feature_type is FeatureType.BBOX:
-            eopatch_content[feature] = _get_common_bbox(eopatches)
+    eopatch_content[(FeatureType.BBOX, None)] = _get_common_bbox(eopatches)
 
     return eopatch_content
 
@@ -138,7 +135,7 @@ def _merge_timestamps(
     """Merges together timestamps from EOPatches. It also prepares a list of masks, one for each EOPatch, how
     timestamps should be ordered and joined together.
     """
-    timestamps_per_eopatch = [eopatch.timestamp for eopatch in eopatches]
+    timestamps_per_eopatch = [eopatch.timestamps for eopatch in eopatches]
     all_timestamps = [timestamp for eopatch_timestamps in timestamps_per_eopatch for timestamp in eopatch_timestamps]
 
     if not all_timestamps:
@@ -166,7 +163,7 @@ def _check_if_optimize(eopatches: Sequence[EOPatch], operation_input: OperationI
     """Checks whether optimisation of `_merge_time_dependent_raster_feature` is possible"""
     if operation_input not in [None, "mean", "median", "min", "max"]:
         return False
-    timestamp_list = [eopatch.timestamp for eopatch in eopatches]
+    timestamp_list = [eopatch.timestamps for eopatch in eopatches]
     return _all_equal(timestamp_list)
 
 
