@@ -2,18 +2,13 @@
 The utilities module is a collection of classes and functions used across the eolearn package, such as checking whether
 two objects are deeply equal, padding of an image, etc.
 
-Credits:
-Copyright (c) 2017-2022 Matej Aleksandrov, Matej Batič, Grega Milčinski, Domagoj Korais, Matic Lubej (Sinergise)
-Copyright (c) 2017-2022 Žiga Lukšič, Devis Peressutti, Tomislav Slijepčević, Nejc Vesel, Jovan Višnjić (Sinergise)
-Copyright (c) 2017-2022 Anže Zupanc (Sinergise)
-Copyright (c) 2019-2020 Jernej Puc, Lojze Žust (Sinergise)
-Copyright (c) 2017-2019 Blaž Sovdat, Andrej Burja (Sinergise)
+Copyright (c) 2017- Sinergise and contributors
+For the full list of contributors, see the CREDITS file in the root directory of this source tree.
 
-This source code is licensed under the MIT license found in the LICENSE
-file in the root directory of this source tree.
+This source code is licensed under the MIT license, see the LICENSE file in the root directory of this source tree.
 """
 import uuid
-from typing import Sequence, Union, cast
+from typing import Callable, Sequence, Tuple, Union, cast
 
 import geopandas as gpd
 import numpy as np
@@ -88,3 +83,39 @@ def generate_uid(prefix: str) -> str:
 def is_discrete_type(number_type: Union[np.dtype, type]) -> bool:
     """Checks if a given `numpy` type is a discrete numerical type."""
     return np.issubdtype(number_type, np.integer) or np.issubdtype(number_type, bool)
+
+
+def _apply_to_spatial_axes(
+    function: Callable[[np.ndarray], np.ndarray], data: np.ndarray, spatial_axes: Tuple[int, int]
+) -> np.ndarray:
+    """Helper function for applying a 2D -> 2D function to a higher dimension array
+
+    Recursively slices data into smaller-dimensional ones, until only the spatial axes remain. The indices of spatial
+    axes have to be adjusted if the recursion-axis is smaller than either one, e.g. spatial axes (1, 2) become (0, 1)
+    after splitting the 3D data along axis 0 into 2D arrays.
+
+    After achieving 2D data slices the mapping function is applied. The data is then reconstructed into original form.
+    """
+
+    ax1, ax2 = spatial_axes
+    if ax1 >= ax2:
+        raise ValueError(
+            f"For parameter `spatial_axes` the second axis must be greater than first, got {spatial_axes}."
+        )
+
+    if ax2 >= data.ndim:
+        raise ValueError(
+            f"Values in `spatial_axes` must be smaller than `data.ndim`, got {spatial_axes} for data of dimension"
+            f" {data.ndim}."
+        )
+
+    if data.ndim <= 2:
+        return function(data)
+
+    axis = next(i for i in range(data.ndim) if i not in spatial_axes)
+    data = np.moveaxis(data, axis, 0)
+
+    ax1, ax2 = (ax if axis > ax else ax - 1 for ax in spatial_axes)
+
+    mapped_slices = [_apply_to_spatial_axes(function, data_slice, (ax1, ax2)) for data_slice in data]
+    return np.moveaxis(np.stack(mapped_slices), 0, axis)
