@@ -2,11 +2,10 @@
 Utilities for a basic Python parallelization that build on top of `concurrent.futures` module from Standard Python
 library.
 
-Credits:
-Copyright (c) 2022 Matej Aleksandrov, Žiga Lukšič (Sinergise)
+Copyright (c) 2017- Sinergise and contributors
+For the full list of contributors, see the CREDITS file in the root directory of this source tree.
 
-This source code is licensed under the MIT license found in the LICENSE
-file in the root directory of this source tree.
+This source code is licensed under the MIT license, see the LICENSE file in the root directory of this source tree.
 """
 import concurrent.futures
 import multiprocessing
